@@ -1,6 +1,6 @@
 /*
  *	basics.h - Basic functionality header 
- *	Copyright (C) 2010, D Haley 
+ *	Copyright (C) 2011, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -40,6 +40,7 @@ class K3DTree;
 
 std::string boolStrEnc(bool b);
 
+bool dummyCallback();
 
 extern const char *DTD_NAME;
 extern const char *PROGRAM_NAME;
@@ -48,6 +49,7 @@ extern const char *FONT_FILE;
 
 
 
+
 //C file peek function
 inline int fpeek(FILE *stream)
 {
@@ -64,14 +66,16 @@ inline int fpeek(FILE *stream)
 //!Text file loader errors
 enum
 {
-	ERR_FILE_INPUT_OPEN_FAIL=1,
-	ERR_FILE_OPEN_FAIL,
-	ERR_FILE_OUT_FAIL,
-	ERR_FILE_FORMAT_FAIL
+	ERR_FILE_OPEN=1,
+	ERR_FILE_FORMAT,
+	ERR_FILE_NUM_FIELDS,
+	ERR_FILE_ENUM_END // not an error, just end of enum
 };
 
+extern const char *TEXT_LOAD_ERR_STRINGS[];
 
-template<class T1, class T2>
+	
+	template<class T1, class T2>
 bool hasFirstInPairVec(const std::vector<std::pair<T1,T2> > &v, const std::pair<T1,T2> &r)
 {
 	for(size_t ui=0;ui<v.size();ui++)
@@ -89,7 +93,15 @@ std::string convertFileStringToNative(const std::string &s);
 std::string convertFileStringToCanonical(const std::string &s);
 
 //!Convert a normal string to a latex one, using charachter replacement
-std::wstring convertToLaTeX(const std::wstring& toConv); 
+void tickSpacingsFromInterspace(float start, float end, 
+		float interSpacing, std::vector<float> &spacings);
+
+void tickSpacingsFromFixedNum(float start, float end, 
+		unsigned int nTicks, std::vector<float> &spacings);
+
+//!Get a "human-like" version of the time elapsed between new and original time period
+std::string veryFuzzyTimeSince( time_t origTime, time_t newTime);
+
 
 //!A routine for loading numeric data from a text file
 unsigned int loadTextData(const char *cpFilename, 
@@ -151,10 +163,6 @@ inline std::wstring stlStrToStlWStr(const std::string& s)
 	return temp;
 }
 
-//performs a string stream cast
-//Returns true on failure
-template<class T1, class T2> bool stream_cast(T1 &result,const T1 &obj);
-
 //!Template function to cast and object to another by the stringstream
 //IO operator
 template<class T1, class T2> bool stream_cast(T1 &result, const T2 &obj)
@@ -273,6 +281,11 @@ int getTotalRAM();
 //!Get available ram in MB
 size_t getAvailRAM();
 
+
+#ifdef DEBUG
+bool isValidXML(const char *filename);
+#endif
+
 inline std::string tabs(unsigned int nTabs)
 {
 	std::string s;
@@ -377,6 +390,11 @@ public:
 
     //! Returns true if any bound is of null thickness
     bool isFlat() const;
+
+    //!Returns true if any bound of datacube is considered to be "large" in magnitude compared to 
+    // floating pt data type.
+    bool isNumericallyBig() const;
+
     //!Obtain bounds from an array of Point3Ds
     void setBounds( const Point3D *ptArray, unsigned int nPoints);
     //!Use two points to set bounds -- does not need to be high,low. this is worked out/
@@ -510,7 +528,7 @@ template<class T> size_t randomSelect(std::vector<T> &result, const std::vector<
 
 	result.resize(num);
 
-	if(strongRandom)
+	if(strongRandom || source.size() < 4)
 	{
 
 		size_t numTicksNeeded;
@@ -682,7 +700,9 @@ template<class T> size_t randomDigitSelection(std::vector<T> &result, const size
 
 	result.resize(num);
 
-	if(strongRandom)
+	//If we have strong randomisation, or we have too few items to use the LFSR,
+	//use proper random generation
+	if(strongRandom || max < 3 )
 	{
 
 		size_t numTicksNeeded;
@@ -698,7 +718,6 @@ template<class T> size_t randomDigitSelection(std::vector<T> &result, const size
 		std::vector<size_t> ticks;
 		ticks.resize(numTicksNeeded);
 
-		std::cerr << "Generating some unique numbers:" << std::endl;
 		//Create an array of numTicksNeededbers and fill 
 		for(size_t ui=0; ui<numTicksNeeded; ui++)
 			ticks[ui]=(size_t)(rng.genUniformDev()*(max-1));
@@ -713,7 +732,6 @@ template<class T> size_t randomDigitSelection(std::vector<T> &result, const size
 		ticks.erase(itLast,ticks.end());
 		std::vector<size_t> moreTicks;
 		//Top up with unique entries
-		std::cerr << "Topping up" << std::endl;
 		while(ticks.size() +moreTicks.size() < numTicksNeeded)
 		{
 			size_t index;
@@ -727,7 +745,6 @@ template<class T> size_t randomDigitSelection(std::vector<T> &result, const size
 				moreTicks.push_back(index);
 
 		}
-		std::cerr << "Finished topup" << std::endl;
 
 		ticks.reserve(numTicksNeeded);
 		for(size_t ui=0;ui<moreTicks.size();ui++)
