@@ -1,6 +1,6 @@
 /*
  *	wxCropPanel.cpp - cropping window  for user interaction
- *	Copyright (C) 2010, D Haley 
+ *	Copyright (C) 2011, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -248,6 +248,8 @@ unsigned int CropPanel::getBestCropWidget(float xMouse, float yMouse,unsigned in
 				x=crop[CROP_LEFT];
 				y=1.0-crop[CROP_BOTTOM];
 				break;
+			default:
+				ASSERT(false);
 		}
 
 		tmpDist=(xMouse-x)*(xMouse-x) + (yMouse-y)*(yMouse-y);
@@ -388,6 +390,8 @@ void CropPanel::mouseDoubleLeftClick(wxMouseEvent& event)
 
 void CropPanel::mouseLeftWindow(wxMouseEvent& event) 
 {
+	if(!dragging)
+		selMode=SELECT_MODE_NONE;
 }
 
 void CropPanel::mouseDown(wxMouseEvent &event)
@@ -459,7 +463,9 @@ void CropPanel::draw()
 	ASSERT(validCoords());
 
 	wxAutoBufferedPaintDC   *dc=new wxAutoBufferedPaintDC(this);
-	dc->SetBackground(*wxWHITE_BRUSH);
+	
+	wxBrush *b = new wxBrush;
+	b->SetColour(wxSystemSettings::GetColour(wxSYS_COLOUR_BACKGROUND));
 	dc->Clear();
 
 	int w,h;
@@ -497,7 +503,8 @@ void CropPanel::draw()
 	//--
 	wxPen *noPen;
 	noPen = new wxPen(*wxBLACK,1,wxTRANSPARENT);
-	dc->SetBrush(*wxLIGHT_GREY);
+	b->SetColour(wxSystemSettings::GetColour(wxSYS_COLOUR_BACKGROUND));
+	dc->SetBrush(*b);
 	dc->SetPen(*noPen);
 
 	dc->DrawRectangle(0,0,lineX[0],h);
@@ -505,6 +512,7 @@ void CropPanel::draw()
 	dc->DrawRectangle(0,lineY[6],w,h-lineY[6]);
 	dc->DrawRectangle(lineX[4],0,w-lineX[4],h);
 	delete noPen;
+	delete b;
 	//--
 	
 	
@@ -566,6 +574,8 @@ void CropPanel::draw()
 				xC=crop[CROP_LEFT];
 				yC=1.0-crop[CROP_BOTTOM];
 				break;
+			default:
+				ASSERT(false);
 		}
 
 		xC=xC*(float)w;
@@ -694,9 +704,5 @@ void CropPanel::onResize(wxSizeEvent &evt)
 #ifndef __WXMAC__
 	wxPaintEvent paintEvt;
 	wxPostEvent(this,paintEvt);
-#else
-	//FIXME: This is not allowable in wx, but seems to be required
-	// for wxMac??
-	draw();
 #endif
 }
