@@ -1,3 +1,22 @@
+/*
+ *	effect.cpp - 3D visuals effects implementation
+ *	Copyright (C) 2011, D Haley 
+
+ *	This program is free software: you can redistribute it and/or modify
+ *	it under the terms of the GNU General Public License as published by
+ *	the Free Software Foundation, either version 3 of the License, or
+ *	(at your option) any later version.
+
+ *	This program is distributed in the hope that it will be useful,
+ *	but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *	GNU General Public License for more details.
+
+ *	You should have received a copy of the GNU General Public License
+ *	along with this program.  If not, see <http://www.gnu.org/licenses/>.
+*/
+
+
 #include "effect.h"
 #include <limits>
 
@@ -169,20 +188,28 @@ void BoxCropEffect::enable(unsigned int pass) const
 			yRot.fz=0;
 			quat_rot(&yRot,&r,angle);
 
-			yTmpRot=Point3D(yRot.fx,yRot.fy,yRot.fy);
+			yTmpRot=Point3D(yRot.fx,yRot.fy,yRot.fz);
 
+			ASSERT(yTmpRot.sqrMag() > sqrtf(std::numeric_limits<float>::epsilon()));
 
 		}
+		else
+			yTmpRot=Point3D(0,1,0);
 
 		//Rotating around the z axis to set "spin"
 		r.fx=z[0];
 		r.fy=z[1];
 		r.fz=z[2];
+
 		angle=y.angle(yTmpRot);
 
-		//Spin the box around to match the final coordinate system
-		for(unsigned int ui=0;ui<8;ui++)
-			quat_rot(&(pBox[ui]),&r,angle);
+
+		if( fabs(angle) > sqrtf(std::numeric_limits<float>::epsilon()))
+		{
+			//Spin the box around to match the final coordinate system
+			for(unsigned int ui=0;ui<8;ui++)
+				quat_rot(&(pBox[ui]),&r,angle);
+		}
 
 
 		//Now compute the box coordinates, then break their position
