@@ -1,6 +1,6 @@
 /*
  *	filter.h - modular data filter implementation 
- *	Copyright (C) 2010, D Haley 
+ *	Copyright (C) 2011, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -52,6 +52,7 @@ const char *STREAM_NAMES[] = { NTRANS("Ion"),
 				NTRANS("Range"),
 				NTRANS("Voxel")};
 
+//Internal names for each filter
 const char *FILTER_NAMES[] = { "posload",
 				"iondownsample",
 				"rangefile",
@@ -65,7 +66,8 @@ const char *FILTER_NAMES[] = { "posload",
 				"spatialanalysis",
 				"clusteranalysis",
 				"voxelise",
-				"ioninfo"
+				"ioninfo",
+				"annotation"
 				};
 
 void updateFilterPropertyGrid(wxPropertyGrid *g, const Filter *f)
@@ -175,13 +177,13 @@ bool parseXMLColour(xmlNodePtr &nodePtr, float &r,float&g,float&b,float&a)
 	return true;
 }
 
-size_t numElements(const vector<const FilterStreamData *> &v, int mask)
+size_t numElements(const vector<const FilterStreamData *> &v, unsigned int mask)
 {
 	size_t nE=0;
 	for(unsigned int ui=0;ui<v.size();ui++)
 	{
 		if((v[ui]->getStreamType() & mask))
-			nE+=v[ui]->GetNumBasicObjects();
+			nE+=v[ui]->getNumBasicObjects();
 	}
 
 	return nE;
@@ -201,7 +203,7 @@ const RangeFile *getRangeFile(const std::vector<const FilterStreamData*> &dataIn
 
 unsigned int getIonstreamIonID(const IonStreamData *d, const RangeFile *r)
 {
-	if(!d->data.size())
+	if(d->data.empty())
 		return (unsigned int)-1;
 
 	unsigned int tentativeRange;
@@ -317,6 +319,19 @@ DrawStreamData::~DrawStreamData()
 	clear();
 }
 
+PlotStreamData::PlotStreamData()
+{
+	streamType=STREAM_TYPE_PLOT;
+	plotType=PLOT_TRACE_LINES;
+	errDat.mode=PLOT_ERROR_NONE;
+	r=1.0,g=0.0,b=0.0,a=1.0;
+	logarithmic=false;
+	index=(unsigned int)-1;
+
+	hardMinX=hardMinY=-std::numeric_limits<float>::max();
+	hardMaxX=hardMaxY=std::numeric_limits<float>::max();
+}
+
 Filter::Filter()
 {
 	cacheOK=false;
@@ -405,3 +420,4 @@ void Filter::initFilter(const std::vector<const FilterStreamData *> &dataIn,
 }
 
 
+
