@@ -6,6 +6,8 @@
 #include <map>
 #include <queue>
 
+#include <gsl/gsl_linalg.h>
+
 enum
 {
 	KEY_CLUSTERANALYSIS_ALGORITHM,
@@ -135,6 +137,55 @@ void makeCompositionTable(const IonStreamData *i ,const RangeFile *r,
 
 }
 
+//Compute the singular vlaues for a matrix of rows "numRows" and "cols"
+// numcols. Matrix to be stored as [curRow*numcols + curCol]
+//If the matrix is rank deficient, then resultValues will have fewer singular
+//values than the full rank matrix (ie < numCols).
+void computeSingularValues(float *data, size_t numRows, size_t numCols,
+				vector<float> &resultValues)
+{
+/*
+	gsl_matrix *m=gsl_matrix_alloc(numRows,numCols);
+	for(unsigned int ui=0;ui<numRows;ui++)
+	{
+		for(unsigned int uj=0;uj<numCols; uj++)
+			gsl_matrix_set(m,ui,uj,data[ui*numCols+uj]);
+	}
+
+	//Set up the output vector space, the singular result space and
+	//some scratch space for the program
+	gsl_vector *singularVals= gsl_vector_alloc(numCols);
+	gsl_vector *workspace = gsl_vector_alloc(numCols);
+	gsl_matrix *newSpace = gsl_matrix_alloc(numCols,numCols);
+
+	//Transformation space set to orthogonal cartesian
+	gsl_matrix_set_identity(newSpace);
+
+	//Decompose matrix. Note that input matrix is overwritten by gsl. 
+	gsl_linalg_SV_decomp(m,newSpace,
+			singularVals,workspace);
+
+	resultValues.clear();
+	for(size_t ui=0;ui<numCols;ui++)
+	{
+		float v;
+		v = gsl_vector_get(singularVals,ui);
+		if(v < sqrt(std::numeric_limits<float>::epsilon()))
+			break;
+
+		resultValues.push_back(v);
+	}
+
+	//Free storage space
+	gsl_vector_free(singularVals); 
+	gsl_vector_free(workspace);
+	
+	gsl_matrix_free(newSpace);
+	gsl_matrix_free(m);
+	*/
+}
+
+
 void ClusterAnalysisFilter::checkIonEnabled(bool &core, bool &bulk) const
 {
 	bulk=core=false;
@@ -186,6 +237,7 @@ ClusterAnalysisFilter::ClusterAnalysisFilter()
 	wantCropSize=false;
 	wantClusterComposition=true;
 	normaliseComposition=true;
+	haveRangeParent=false; //we set this to false, as required by ::initFilter
 
 	cacheOK=false;
 
@@ -511,7 +563,6 @@ unsigned int ClusterAnalysisFilter::refresh(const std::vector<const FilterStream
 	}
 
 
-
 	//Construct the output clustered data.
 	IonStreamData *i = new IonStreamData;
 	i->parent =this;	
@@ -770,6 +821,7 @@ void ClusterAnalysisFilter::getProperties(FilterProperties &propertyList) const
 		keys.push_back(KEY_NORMALISE_COMPOSITION);
 	}
 
+
 	propertyList.data.push_back(s);
 	propertyList.types.push_back(type);
 	propertyList.keys.push_back(keys);
@@ -1330,8 +1382,9 @@ bool ClusterAnalysisFilter::readState(xmlNodePtr &nodePtr, const std::string &pa
 				return false;
 			if(!XMLGetNextElemAttrib(nodePtr,coreKNN,"coringknn","value"))
 				return false;
-			if(coreKNN<=0)
+			if(!coreKNN)
 				return false;
+
 			if(!XMLGetNextElemAttrib(nodePtr,linkDist,"linkdist","value"))
 				return false;
 			if(linkDist<=0)
@@ -1417,13 +1470,13 @@ bool ClusterAnalysisFilter::readState(xmlNodePtr &nodePtr, const std::string &pa
 	return true;
 }
 
-int ClusterAnalysisFilter::getRefreshBlockMask() const
+unsigned int ClusterAnalysisFilter::getRefreshBlockMask() const
 {
 	//Anything but ions can go through this filter.
 	return STREAM_TYPE_IONS;
 }
 
-int ClusterAnalysisFilter::getRefreshEmitMask() const
+unsigned int ClusterAnalysisFilter::getRefreshEmitMask() const
 {
 	if(wantClusterSizeDist || wantClusterComposition)
 		return STREAM_TYPE_IONS | STREAM_TYPE_PLOT;
@@ -1538,7 +1591,7 @@ unsigned int ClusterAnalysisFilter::refreshLinkClustering(const std::vector<cons
 	vector<IonHit> coreIons,bulkIons;
 	createRangedIons(dataIn,coreIons,bulkIons,progress,callback);
 
-	if(!coreIons.size())
+	if(coreIons.empty())
 		return 0;
 	//----------
 
@@ -1734,7 +1787,7 @@ unsigned int ClusterAnalysisFilter::refreshLinkClustering(const std::vector<cons
 						*thisPt,bCore, true);
 
 
-				ASSERT(clustIdx == (unsigned int)-1 || coreTree.getTag(clustIdx));
+				ASSERT(clustIdx == (size_t)-1 || coreTree.getTag(clustIdx));
 				if(clustIdx != (size_t)-1)
 				{
 					curDistSqr=coreTree.getPt(clustIdx)->sqrDist(
@@ -2549,102 +2602,73 @@ void ClusterAnalysisFilter::genCompositionVersusSize(const vector<vector<IonHit>
 
 }
 
-/* I think this is roughly OK (written, but not compiled/tested), but we are going to get a code explosion, as
- * I have to enumerate all the differeing paths, such that I don't have to
- * manually *check* each boolean option (count bulk/no count bulk, use paralell/use single)
- * So this is disabled for now.
-void ClusterAnalysisFilter::makeRangedFreqMap(const <vector<vector<IonHIT> > &clusteredCore,
-			const vector<vector<IonHit> > &clusteredBulk, const RangeFile *rng.
-			map<size_t,vector<size_t> > countMap)
+void ClusterAnalysisFilter::getSingularValues(const vector<vector<IonHit> > &clusteredCore,
+				const vector<vector<IonHit> > &clusteredBulk, vector<vector<float> > &singularValues)
 {
-	vector<size_t> dummyIonFreqTable;
-	dummyInFreqTable.resize(rng->getNumIons(),0);	
+	ASSERT(clusteredCore.size() == clusteredBulk.size());
 
-#ifdef OPENMP
-	//parallel version
-	//---
-	vector< map<size_t,vector<size_t> > > countMaps;
-	unsigned int maxThreads=omp_get_max_num_threads();
-	
-	//OK, so the composition will be the core and bulk both;
-	//Now, lets figure out who belongs where.
-	#pragma omp parallel for
-	for(size_t ui=0;ui<clusteredCore.size();ui++)
+	vector<float> curSingularVals;
+	float *data=0;
+	size_t dataSize=0;
+	for(unsigned int ui=0;ui<clusteredCore.size(); ui++)
 	{
-		unsigned int thread = omp_get_thread_num();
-		if(countMaps[thread].find(curSize) ==countMaps.end())
-			countMaps[thread].insert(make_pair(curSize,dummyIonFreqTable));
-	}
-	
-	#pragma omp parallel for 	
-	for(size_t ui=0;ui<clusteredCore.size();ui++)
-	{
-		unsigned int thread = omp_get_thread_num();
+
+		curSingularVals.clear();
+		//For this cluster, compute its singular values amplitudes
+		// (ignore direction)
+		const unsigned int DIMENSION=3;
+		size_t numEntries = clusteredCore[ui].size() + clusteredBulk[ui].size();
+
+
+		//Check to see if we have sufficient data to compute an SVD
+		if(numEntries < DIMENSION )
+		{
+			singularValues.push_back(curSingularVals);
+			continue; 
+		}
+		
+		//Allocate space, by growing as needed
+		if(numEntries*DIMENSION > dataSize)
+		{
+			//TODO: Is it faster to pre-compute the maximum cluster size?
+			//      then determine the cluster backbone from that? 
+			//      Should we be usig realloc?
+			if(data)
+				delete[] data;
+			data = new float[numEntries*DIMENSION];
+			dataSize=numEntries*DIMENSION;
+		}
+
+		//Fill the data array
+		float *p;
+		p=data;
 		for(size_t uj=0;uj<clusteredCore[ui].size();uj++)
 		{
-			offset= rng->getIonID(clusteredCore[ui][uj]);
-			countMaps[thread][curSize][offset]++;
+			for(size_t uk=0;uk<DIMENSION;uk++)
+			{
+				*(p) = clusteredCore[ui][uj].getPos()[uk];
+				p++;
+			}
 		}
 		
 		for(size_t uj=0;uj<clusteredBulk[ui].size();uj++)
 		{
-			offset= rng->getIonID(clusteredBulk[ui][uj]);
-			countMaps[thread][curSize][offset]++;
+			for(size_t uk=0;uk<DIMENSION;uk++)
+			{
+				*(p) = clusteredBulk[ui][uj].getPos()[uk];
+				p++;
+			}
 		}
 
+		//Cmpute the DIMENSION-D singular value decomposition for
+		//this vector set
+		computeSingularValues(data,numEntries,
+					DIMENSION,curSingularVals);
+
+		singularValues.push_back(curSingularVals);
 	}
 
-
-	//merge the differing threads into a single map
-	//which is our final frequency tally
-
-
-	//firstly we have to "prepare" the final "master" map.
-	//ie, fill it with zeros at the appropriate sizes.
-	for(size_t ui=0;ui<countMaps.size();ui++)
-	{
-		//Loop through the map, and prepare the final "master" map with
-		//zero values
-		for(map<size_t,vector<size_t> >::const_iterator 
-			it=countMaps.begin(); it!=countMaps.end();it++)
-		{
-			if(countMap.find(it->first) == countMaps.end())
-				countMap.insert(it->first,dummyIonFreqTable);
-		}
-	}
-
-	//Now we have to sum the thread-specific maps into the master map
-	for(size_t ui=0;ui<countMaps.size();ui++)
-	{
-		for(map<size_t,vector<size_t> >::const_iterator 
-			it=countMaps.begin(); it!=countMaps.end();it++)
-			countMap[it->first]+=it->second;
-	}
-	//---
-#else
-	//Single-threaded version
-	//---
-	for(size_t ui=0;ui<clusteredCore.size();ui++)
-	{
-		if(countMap.find(curSize) ==countMap.end())
-			countMap.insert(make_pair(curSize,ionFreq));
-	}
-	
-	for(size_t ui=0;ui<clusteredCore.size();ui++)
-	{
-		for(size_t uj=0;uj<clusteredCore[ui].size();uj++)
-		{
-			offset= rng->getIonID(clusteredCore[ui][uj]);
-			countMap[curSize][offset]++;
-		}
-		
-		for(size_t uj=0;uj<clusteredBulk[ui].size();uj++)
-		{
-			offset= rng->getIonID(clusteredBulk[ui][uj]);
-			countMap[curSize][offset]++;
-		}
-
-	}
-#endif
+	if(data)
+		delete[] data;
 }
- */
+
