@@ -105,7 +105,6 @@ Filter *CompositionProfileFilter::cloneUncached() const
 	std::copy(scalarParams.begin(),scalarParams.end(),p->scalarParams.begin());
 
 	p->normalise=normalise;	
-	p->stepMode=stepMode;	
 	p->fixedBins=fixedBins;
 	p->lockAxisMag=lockAxisMag;
 
@@ -383,7 +382,7 @@ unsigned int CompositionProfileFilter::refresh(const std::vector<const FilterStr
 
 						float angle = dir.angle(direction);
 
-						float halfLen=sqrt(vectorParams[1].sqrMag())/2.0f;
+						float halfLen=sqrt(vectorParams[1].sqrMag());
 						float sqrRad=scalarParams[0]*scalarParams[0];
 				
 						//Check that we actually need to rotate, to avoid numerical singularity 
@@ -554,6 +553,17 @@ unsigned int CompositionProfileFilter::refresh(const std::vector<const FilterStr
 		}
 
 		plotData[ui]->xyData.resize(ionFrequencies[ui].size());
+	
+		//Density profiles (non-ranged plots) have a fixed normalisation factor
+		if(!rngData && normalise)
+		{
+			if(fixedBins)
+				normFactor = 1.0/(M_PI*scalarParams[0]*scalarParams[0]*(length/(float)numBins));
+			else
+				normFactor = 1.0/(M_PI*scalarParams[0]*scalarParams[0]*binWidth);
+		}	
+
+		//Go through each bin, then perform the appropriate normalisation
 		for(unsigned int uj=0;uj<ionFrequencies[ui].size(); uj++)
 		{
 			float xPos;
@@ -579,15 +589,7 @@ unsigned int CompositionProfileFilter::refresh(const std::vector<const FilterStr
 			}
 			else
 			{
-				if(normalise)
-				{
-					//This is a frequency profile. Normalising across bins would lead to every value
-					//being equal to 1. Let us instead normalise by cylinder section volume)
-					normFactor = 1.0/(M_PI*scalarParams[0]*scalarParams[0]*binWidth);
-
-
-				}
-				//Normalise Y value against volume of bin
+				//This is a frequency profile (factor ==1), or density profile (factor computed above).
 				plotData[ui]->xyData[uj] = std::make_pair(
 					xPos,normFactor*(float)ionFrequencies[ui][uj]);
 
@@ -738,7 +740,7 @@ bool CompositionProfileFilter::setProperty(unsigned int set, unsigned int key,
 			scalarParams.clear();
 			switch(primitiveType)
 			{
-				case IONCLIP_PRIMITIVE_CYLINDER:
+				case COMPPROFILE_PRIMITIVE_CYLINDER:
 					vectorParams.push_back(Point3D(0,0,0));
 					vectorParams.push_back(Point3D(0,20,0));
 					scalarParams.push_back(10.0f);
@@ -1087,7 +1089,6 @@ bool CompositionProfileFilter::writeState(std::ofstream &f,unsigned int format,
 			
 			f << tabs(depth+1) << "</scalarparams>" << endl;
 			f << tabs(depth+1) << "<normalise value=\"" << normalise << "\"/>" << endl;
-			f << tabs(depth+1) << "<stepmode value=\"" << stepMode << "\"/>" << endl;
 			f << tabs(depth+1) << "<fixedbins value=\"" << (int)fixedBins << "\"/>" << endl;
 			f << tabs(depth+1) << "<nbins value=\"" << nBins << "\"/>" << endl;
 			f << tabs(depth+1) << "<binwidth value=\"" << binWidth << "\"/>" << endl;
@@ -1300,23 +1301,6 @@ bool CompositionProfileFilter::readState(xmlNodePtr &nodePtr, const std::string
 	xmlFree(xmlString);
 	//====
 
-	//Retrieve step mode 
-	//====
-	if(XMLHelpFwdToElem(nodePtr,"stepmode"))
-		return false;
-
-	xmlString=xmlGetProp(nodePtr,(const xmlChar *)"value");
-	if(!xmlString)
-		return false;
-	tmpStr=(char *)xmlString;
-
-	if(stream_cast(stepMode,tmpStr))
-		return false;
-
-	xmlFree(xmlString);
-	//====
-
-
 	//Retrieve fixed bins on/off 
 	//====
 	if(XMLHelpFwdToElem(nodePtr,"fixedbins"))
@@ -1400,13 +1384,13 @@ bool CompositionProfileFilter::readState(xmlNodePtr &nodePtr, const std::string
 	return true;
 }
 
-int CompositionProfileFilter::getRefreshBlockMask() const
+unsigned int CompositionProfileFilter::getRefreshBlockMask() const
 {
 	//Absolutely anything can go through this filter.
 	return 0;
 }
 
-int CompositionProfileFilter::getRefreshEmitMask() const
+unsigned int CompositionProfileFilter::getRefreshEmitMask() const
 {
 	if(showPrimitive)
 		return STREAM_TYPE_PLOT | STREAM_TYPE_DRAW;
