@@ -297,7 +297,7 @@ unsigned int VoxeliseFilter::refresh(const std::vector<const FilterStreamData *>
 
 		const IonStreamData *is = (const IonStreamData *)dataIn[i];
 		//Don't work on empty or single object streams (bounding box needs to be defined)
-		if (is->GetNumBasicObjects() < 2) continue;
+		if (is->getNumBasicObjects() < 2) continue;
 	
 		BoundCube bcTmp;
 		bcTmp=getIonDataLimits(is->data);
@@ -471,7 +471,7 @@ unsigned int VoxeliseFilter::refresh(const std::vector<const FilterStreamData *>
 				return VOXELISE_MEMORY_ERR; 
 
 			//Gaussian kernel is separable (rank 1)
-			if(vs->data.separableConvolve(kernel,res,false,modeMap[filterBoundaryMode]))
+			if(vs->data.separableConvolve(kernel,res,modeMap[filterBoundaryMode]))
 				return VOXELISE_CONVOLVE_ERR;
 
 			vs->data.swap(res);
@@ -510,6 +510,7 @@ unsigned int VoxeliseFilter::refresh(const std::vector<const FilterStreamData *>
 	return 0;
 }
 
+//TODO: Use arrays
 std::string VoxeliseFilter::getNormaliseTypeString(int type) const {
 	switch (type) {
 		case VOXELISE_NORMALISETYPE_NONE:
@@ -852,7 +853,7 @@ bool VoxeliseFilter::setProperty( unsigned int set, unsigned int key,
 			 unsigned int i;
 			if(stream_cast(i,value))
 				return false;
-			if(i <= 0)
+			if(!i)
 				return false;
 
 			needUpdate=true;
@@ -873,7 +874,7 @@ bool VoxeliseFilter::setProperty( unsigned int set, unsigned int key,
 			 unsigned int i;
 			if(stream_cast(i,value))
 				return false;
-			if(i <= 0)
+			if(!i)
 				return false;
 			needUpdate=true;
 			//if the result is different, the
@@ -892,7 +893,7 @@ bool VoxeliseFilter::setProperty( unsigned int set, unsigned int key,
 			 unsigned int i;
 			if(stream_cast(i,value))
 				return false;
-			if(i <= 0)
+			if(!i)
 				return false;
 			
 			//if the result is different, the
@@ -958,8 +959,8 @@ bool VoxeliseFilter::setProperty( unsigned int set, unsigned int key,
 		}
 		case KEY_VOXELISE_NORMALISE_TYPE:
 		{
-			 unsigned int i;
-			for (i = 0; i < VOXELISE_NORMALISETYPE_MAX; i++)
+			unsigned int i;
+			for(i = 0; i < VOXELISE_NORMALISETYPE_MAX; i++)
 				if (value == getNormaliseTypeString(i)) break;
 			if (i == VOXELISE_NORMALISETYPE_MAX)
 				return false;
@@ -1163,7 +1164,7 @@ bool VoxeliseFilter::setProperty( unsigned int set, unsigned int key,
 				return false;
 
 			//FIXME: Min restriction is artificial and imposed due to incomplete separerable convolution filter implementation
-			if(i <= 0 || i > std::min(nBins[0],std::min(nBins[1],nBins[2])))
+			if(i == 0 || i > std::min(nBins[0],std::min(nBins[1],nBins[2])))
 				return false;
 			if(i != filterBins)
 			{
@@ -1441,13 +1442,13 @@ bool VoxeliseFilter::readState(xmlNodePtr &nodePtr, const std::string &stateFile
 	
 }
 
-int VoxeliseFilter::getRefreshBlockMask() const
+unsigned int VoxeliseFilter::getRefreshBlockMask() const
 {
 	//Ions, plots and voxels cannot pass through this filter
 	return STREAM_TYPE_IONS | STREAM_TYPE_PLOT | STREAM_TYPE_VOXEL;
 }
 
-int VoxeliseFilter::getRefreshEmitMask() const
+unsigned int VoxeliseFilter::getRefreshEmitMask() const
 {
 	return STREAM_TYPE_VOXEL | STREAM_TYPE_DRAW;
 }
