@@ -1,6 +1,6 @@
 /*
  *	glPane.cpp - OpenGL panel implementation
- *	Copyright (C) 2010, D Haley 
+ *	Copyright (C) 2011, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -46,6 +46,14 @@
 #endif
 
 
+enum
+{
+	ID_KEYPRESS_TIMER=wxID_ANY+1,
+};
+
+//Double tap delay (ms), for axis reversal
+const unsigned int DOUBLE_TAP_DELAY=500; 
+
 BEGIN_EVENT_TABLE(BasicGLPane, wxGLCanvas)
 EVT_MOTION(BasicGLPane::mouseMoved)
 EVT_ERASE_BACKGROUND(BasicGLPane::OnEraseBackground)
@@ -62,6 +70,7 @@ EVT_CHAR(BasicGLPane::charEvent)
 EVT_KEY_UP(BasicGLPane::keyReleased)
 EVT_MOUSEWHEEL(BasicGLPane::mouseWheelMoved)
 EVT_PAINT(BasicGLPane::render)
+EVT_TIMER(ID_KEYPRESS_TIMER,BasicGLPane::OnAxisTapTimer)
 END_EVENT_TABLE()
 
 //Controls camera pan/translate/pivot speed; Radii per pixel or distance/pixel
@@ -69,6 +78,8 @@ const float CAMERA_MOVE_RATE=0.05;
 
 // Controls zoom speed, in err, zoom units.. Ahem. 
 const float CAMERA_SCROLL_RATE=0.05;
+//Zoom speed for keyboard
+const float CAMERA_KEYBOARD_SCROLL_RATE=1;
 
 int attribList[] = {WX_GL_RGBA,
 			WX_GL_DEPTH_SIZE,
@@ -84,12 +95,34 @@ wxGLCanvas(parent, wxID_ANY,  wxDefaultPosition, wxDefaultSize, 0, wxT("GLCanvas
 	applyingDevice=false;
 	paneInitialised=false;
 
+	keyDoubleTapTimer=new wxTimer(this,ID_KEYPRESS_TIMER);
+	lastKeyDoubleTap=(unsigned int)-1;
+
 	mouseMoveFactor=mouseZoomFactor=1.0f;	
 	dragging=false;
 	lastMoveShiftDown=false;
 	selectionMode=false;
 }
 
+BasicGLPane::~BasicGLPane()
+{
+	keyDoubleTapTimer->Stop();
+	delete keyDoubleTapTimer;
+}
+
+bool BasicGLPane::displaySupported() const
+{
+#if wxCHECK_VERSION(2,9,0)
+	return IsDisplaySupported(attribList);
+#else
+	ASSERT(false);
+	//Lets hope so. If its not, then its just going to fai anyway. 
+	//If it is, then returning false would simply create a roadblock.
+	//Either way, you shouldn't get here.
+	return true; 
+#endif
+}
+
 unsigned int  BasicGLPane::selectionTest(wxPoint &p,bool &shouldRedraw)
 {
 
@@ -450,7 +483,10 @@ void BasicGLPane::mouseReleased(wxMouseEvent& event)
 	
 }
 
-void BasicGLPane::rightClick(wxMouseEvent& event) {}
+void BasicGLPane::rightClick(wxMouseEvent& event) 
+{
+}
+
 void BasicGLPane::mouseLeftWindow(wxMouseEvent& event) 
 {
 	if(selectionMode)
@@ -491,29 +527,88 @@ void BasicGLPane::mouseLeftWindow(wxMouseEvent& event)
 
 void BasicGLPane::keyPressed(wxKeyEvent& event) 
 {
+	
 	switch(event.GetKeyCode())
 	{
 		case WXK_SPACE:
 		{
 			unsigned int visibleDir;
+	
 			//Use modifier keys to alter the direction of visiblity
-			if(event.CmdDown())
-				visibleDir= 0;
-			else if (event.ShiftDown())
-				visibleDir=2;
-			else
-				visibleDir=3;
-			currentScene.ensureVisible(visibleDir);
+			//First compute the part of the keymask that does not
+			//reflect the double tap
+			unsigned int keyMask;
+			keyMask = (event.CmdDown() ? 1 : 0);
+			keyMask |= (event.ShiftDown() ?  2 : 0);
+
+			//Now determine if we are the same mask as last time
+			bool isKeyDoubleTap=(lastKeyDoubleTap==keyMask);
+			//double tapping allows for selection of reverse direction
+			keyMask |= ( isKeyDoubleTap ?  4 : 0);
+			
+			visibleDir=-1;
+
+			//Hardwire key combo->Mapping
+			switch(keyMask)
+			{
+				//Space only
+				case 0: 
+					visibleDir=3; 
+					break;
+				//Command down +space 
+				case 1:
+					visibleDir=0;
+					break;
+				//Shift +space 
+				case 2:
+					visibleDir=2;
+					break;
+				//NO CASE 3	
+				//Double+space
+				case 4:
+					visibleDir=5; 
+					break;
+				//Doublespace+Cmd
+				case 5:
+					visibleDir=4;
+					break;
+
+				//Space+Double+shift
+				case 6:
+					visibleDir=1;
+					break;
+				default:
+					;
+			}
+
+			if(visibleDir!=(unsigned int)-1)
+			{
+
+				if(isKeyDoubleTap)
+				{
+					//It was a double tap. Reset the tapping and stop the timer
+					lastKeyDoubleTap=(unsigned int)-1;
+					keyDoubleTapTimer->Stop();
+				}
+				else
+				{
+					lastKeyDoubleTap=keyMask & (~(0x04));
+					keyDoubleTapTimer->Start(DOUBLE_TAP_DELAY,wxTIMER_ONE_SHOT);
+				}
+
+				
+				currentScene.ensureVisible(visibleDir);
 #ifdef wxMAC
-			parentStatusBar->SetStatusText(wxTRANS("Use shift/⌘-space to alter reset axis"));
+				parentStatusBar->SetStatusText(wxTRANS("Use shift/⌘-space or double tap to alter reset axis"));
 #else
-			parentStatusBar->SetStatusText(wxTRANS("Use shift/ctrl-space to alter reset axis"));
+				parentStatusBar->SetStatusText(wxTRANS("Use shift/ctrl-space or double tap to alter reset axis"));
 #endif
-			parentStatusBar->SetBackgroundColour(*wxCYAN)
-				;
-			parentStatusTimer->Start(statusDelay,wxTIMER_ONE_SHOT);
-			Refresh();
-			haveCameraUpdates=true;
+				parentStatusBar->SetBackgroundColour(*wxCYAN)
+					;
+				parentStatusTimer->Start(statusDelay,wxTIMER_ONE_SHOT);
+				Refresh();
+				haveCameraUpdates=true;
+			}
 		}
 		break;
 	}
@@ -536,9 +631,8 @@ void BasicGLPane::setGlClearColour(float r, float g, float b)
 
 void BasicGLPane::keyReleased(wxKeyEvent& event) 
 {
-	const float CAMERA_SCROLL_RATE=1;
 
-	float cameraMoveRate=CAMERA_SCROLL_RATE;
+	float cameraMoveRate=CAMERA_KEYBOARD_SCROLL_RATE;
 
 	if(event.ShiftDown())
 		cameraMoveRate*=5;
@@ -576,6 +670,7 @@ void BasicGLPane::keyReleased(wxKeyEvent& event)
 	Refresh();
 	event.Skip();
 }
+
 void BasicGLPane::charEvent(wxKeyEvent& event) 
 {
 
@@ -702,6 +797,8 @@ void BasicGLPane::updateClearColour()
 {
 	float rClear,gClear,bClear;
 	currentScene.getBackgroundColour(rClear,gClear,bClear);
+	//Can't set the opengl window without a proper context
+	ASSERT(paneInitialised);
 	setGlClearColour(rClear,gClear,bClear);
 	//Let openGL know that we have changed the colour.
 	glClearColor( rClear, gClear, 
@@ -889,6 +986,11 @@ bool BasicGLPane::saveImage(unsigned int width, unsigned int height,
 	return isOK;
 }
 
+void BasicGLPane::OnAxisTapTimer(wxTimerEvent &evt)
+{
+	lastKeyDoubleTap=(unsigned int)-1;
+}
+
 
 bool BasicGLPane::saveImageSequence(unsigned int resX, unsigned int resY, unsigned int nFrames,
 		wxString &path,wxString &prefix, wxString &ext)
