@@ -1,6 +1,6 @@
 /*
- *	mathfuncs.h - Miscellaneous mathematical functions implementation. 
- *	Copyright (C) 2010, D Haley 
+ *	mathfuncs.cpp - Miscellaneous mathematical functions implementation. 
+ *	Copyright (C) 2011, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -216,6 +216,22 @@ float Point3D::angle(const Point3D &pt) const
 	return acos(dotProd(pt)/(sqrtf(sqrMag()*pt.sqrMag())));
 }
 
+bool Point3D::orthogonalise(const Point3D &pt)
+{
+	Point3D crossp;
+	crossp=this->crossProd(pt);
+
+	//They are co-linear, or near-enough to be not resolvable.
+	if(crossp.sqrMag()  < sqrt(std::numeric_limits<float>::epsilon()))
+		return false;
+	crossp.normalise();
+
+	crossp=crossp.crossProd(pt);
+	*this=crossp.normalise()*sqrt(this->sqrMag());	
+
+	return true;
+}
+
 #ifdef __LITTLE_ENDIAN__
 
 void Point3D::switchEndian()
@@ -405,6 +421,9 @@ void quat_pointmult(Point3f *result, Quaternion *q1, Quaternion *q2)
 //Note result is stored in returned point
 void quat_rot(Point3f *point, Point3f *rotVec, float angle)
 {
+	ASSERT(rotVec->fx*rotVec->fx + rotVec->fy*rotVec->fy + rotVec->fz*rotVec->fz - 1.0f < 
+			5.0f*sqrt(std::numeric_limits<float>::epsilon()));
+
 	double sinCoeff;
        	Quaternion rotQuat;
 	Quaternion pointQuat;
@@ -442,6 +461,8 @@ void quat_rot(Point3f *point, Point3f *rotVec, float angle)
 //Retrieve the quaternion for repeated rotation. pass to the quat_rot_apply_quats
 void quat_get_rot_quat(Point3f *rotVec, float angle,Quaternion *rotQuat) 
 {
+	ASSERT(rotVec->fx*rotVec->fx + rotVec->fy*rotVec->fy + rotVec->fz*rotVec->fz - 1.0f < 
+			5.0f*sqrt(std::numeric_limits<float>::epsilon()));
 	double sinCoeff;
 #ifdef _GNU_SOURCE
 	double cosCoeff;
