@@ -1,6 +1,6 @@
 /*
  *	scene.cpp  - OPengl scene implementation
- *	Copyright (C) 2010, D Haley 
+ *	Copyright (C) 2011, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -294,6 +294,8 @@ void Scene::drawOverlays() const
 	glMatrixMode(GL_PROJECTION);
 	glPushMatrix();
 	glLoadIdentity();
+
+	glDisable(GL_LIGHTING);
 	//Set the opengl camera state back into modelview mode
 	if(viewRestrict)
 	{
@@ -587,7 +589,7 @@ void Scene::removeCam(unsigned int camIDVal)
 	cameras.erase(cameras.begin()+position);
 	camIDs.killByPos(position);
 
-	if(cameras.size() >0)
+	if(cameras.size())
 	{
 		activeCam=0;
 		cameraSet=true;
