@@ -1,6 +1,6 @@
 /*
  *	viscontrol.cpp - visualation-user interface glue code
- *	Copyright (C) 2010, D Haley 
+ *	Copyright (C) 2011, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -22,6 +22,11 @@
 #include <list>
 #include <stack>
 
+#ifdef DEBUG
+//Only needed for assertion check
+#include <cstdlib>
+#endif
+
 #include "scene.h"
 #include "drawables.h"
 
@@ -186,25 +191,30 @@ void VisController::setYieldWindow(wxWindow *newYield)
 	yieldWindow=newYield;
 }
 
-unsigned int VisController::LoadIonSet(const std::string &filename)
+unsigned int VisController::LoadIonSet(const std::string &filename, Filter *filtPos,Filter *down)
 {
 
 	//Test to see if we can open the file
 	ifstream f(filename.c_str());
 
 	if(!f)
+	{
+		delete filtPos;
+		delete down;
 		return POS_OPEN_FAIL;
+	}
 
 	f.close();
 
+	ASSERT(filtPos->getType() == FILTER_TYPE_POSLOAD);
+	ASSERT(down->getType() == FILTER_TYPE_IONDOWNSAMPLE);
+	DataLoadFilter *p=(DataLoadFilter*)filtPos;
 	//Save current filter state to undo stack
 	pushUndoStack();
 
-
-	DataLoadFilter *p = new DataLoadFilter();
-
 	p->setFilename(filename);
 
+
 	tree<Filter *>::iterator it;
 	//From version 1.45 of ttree.h the top element should be added
 	// using 'insert'.	
@@ -213,8 +223,9 @@ unsigned int VisController::LoadIonSet(const std::string &filename)
 	else
 		it=filters.insert_after(filters.begin(),p);
 
+	IonDownsampleFilter *d=(IonDownsampleFilter*)down;
 	//Append a new filter to the filter tree
-	filters.append_child(it,new IonDownsampleFilter);
+	filters.append_child(it,d);
 
 	return 0;
 }
@@ -599,7 +610,6 @@ void VisController::initFilterTree() const
 	}
 }
 
-
 void VisController::getFilterRefreshStarts(vector<tree<Filter *>::iterator > &propStarts) const
 {
 
@@ -636,12 +646,10 @@ void VisController::getFilterRefreshStarts(vector<tree<Filter *>::iterator > &pr
 			if(!filters.is_valid(it))
 				break;
 
-			ASSERT(filters.is_valid(it));
-			
 			size_t curEmit;
 			//Root node is special, does not combine with the 
 			//previous filter
-			if(!filters.depth(it)) //TODO: Checkme, 0 or 1?
+			if(!filters.depth(it)) 
 				curEmit=(*it)->getRefreshEmitMask();
 			else
 			{
@@ -659,6 +667,7 @@ void VisController::getFilterRefreshStarts(vector<tree<Filter *>::iterator > &pr
 
 		//Build the accumulated block map;  this describes
 		//what types, if emitted, will NOT be propagated to the final output
+		//Nor affect any downstream filters
 		for(size_t ui=filters.max_depth()+1; ui; )
 		{
 			ui--;
@@ -670,15 +679,24 @@ void VisController::getFilterRefreshStarts(vector<tree<Filter *>::iterator > &pr
 					continue;
 
 
-				int blockMask=0xFFFF;
+				//Initially assume that everything is passed through
+				//filter
+				int blockMask=0x0;
+
 
 				if((*it)->haveCache())
 				{
 					//Loop over the children of this filter, grab their block masks
 					for(tree<Filter *>::sibling_iterator itJ=it.begin(); itJ!=it.end();itJ++)
 					{
+
 						if((*itJ)->haveCache())
-							blockMask&=(*itJ)->getRefreshBlockMask();
+						{
+							int curBlockMask;
+							curBlockMask=(*itJ)->getRefreshBlockMask();
+							blockMask= (blockMask & curBlockMask);
+
+						}
 						else
 						{
 							blockMask&=0;
@@ -760,8 +778,13 @@ void VisController::getFilterRefreshStarts(vector<tree<Filter *>::iterator > &pr
 			for(tree<Filter *>::sibling_iterator itJ=filters.begin(it); itJ!=filters.end(it);itJ++)
 				blockMask&=accumulatedBlockTypes[*itJ];
 
+
+			 
+
+
 			if( emitMask & ((~blockMask) & STREAMTYPE_MASK_ALL)) 
 			{
+
 				//Oh noes! we don't block, we will have to stop here,
 				// for this subtree. We cannot go further down.
 				seedFilts.push_back(it);
@@ -769,10 +792,7 @@ void VisController::getFilterRefreshStarts(vector<tree<Filter *>::iterator > &pr
 
 		}
 
-
 		propStarts.swap(seedFilts);
-
-
 	}
 
 #ifdef DEBUG
@@ -789,9 +809,6 @@ void VisController::getFilterRefreshStarts(vector<tree<Filter *>::iterator > &pr
 				!isChild(filters,propStarts[uj],propStarts[ui]));
 		}
 	}
-
-
-
 #endif
 
 
@@ -812,6 +829,29 @@ unsigned int VisController::refreshFilterTree(list<std::pair<Filter *,vector<con
 		return 0;	
 	}
 
+	//Destroy any caches that belong to monitored filters that need
+	//refreshing. Failing to do this can lead to filters being skipped
+	//during the refresh 
+	for(tree<Filter *>::iterator filterIt=filters.begin();
+			filterIt!=filters.end();++filterIt)
+	{
+		//We need to clear the cache of *all* 
+		//downstream filters, as otherwise
+		//their cache's could block our update.
+		if((*filterIt)->monitorNeedsRefresh())
+		{
+			for(tree<Filter *>::pre_order_iterator it(filterIt);it!= filters.end(); ++it)
+			{
+				//Do not traverse siblings
+				if(filters.depth(filterIt) >= filters.depth(it) && it!=filterIt )
+					break;
+			
+				(*it)->clearCache();
+			}
+		}
+	}
+
+
 	// -- Build data streams --	
 	vector< const FilterStreamData *> curData;
 	stack<vector<const FilterStreamData * > > inDataStack;
@@ -844,7 +884,7 @@ unsigned int VisController::refreshFilterTree(list<std::pair<Filter *,vector<con
 		{
 			//Depth-first search from root node, refreshing filters as we proceed.
 			for(tree<Filter * >::pre_order_iterator filtIt=baseTreeNodes[itPos];
-							filtIt!=filters.end(); filtIt++)
+							filtIt!=filters.end(); ++filtIt)
 			{
 				//Check to see if this node is a child of the base node.
 				//if not, move on.
@@ -1075,6 +1115,21 @@ unsigned int VisController::refreshFilterTree(list<std::pair<Filter *,vector<con
 	return 0;
 }
 
+bool VisController::hasUpdates() const
+{
+	if(pendingUpdates)
+		return true;
+
+	for(tree<Filter *>::iterator it=filters.begin();it!=filters.end();++it)
+	{
+		if((*it)->monitorNeedsRefresh())
+			return true;
+	}
+
+	return false;
+}
+
+
 void VisController::getFilterUpdates()
 {
 	vector<pair<const Filter *,SelectionBinding> > bindings;
@@ -1188,8 +1243,12 @@ unsigned int VisController::updateScene()
 								ionData->a);
 					//set the size from the ionstream data
 					curIonDraw->setSize(ionData->ionSize);
+				
+					(*wxYieldCallback)();
 					//Randomly shuffle the ion data before we draw it
 					curIonDraw->shuffle();
+					
+					(*wxYieldCallback)();
 
 					
 					drawIons.push_back(curIonDraw);
@@ -1212,7 +1271,7 @@ unsigned int VisController::updateScene()
 					plotData=((PlotStreamData *)((*it)[ui]));
 					
 					//The plot should have some data in it.
-					ASSERT(plotData->GetNumBasicObjects());
+					ASSERT(plotData->getNumBasicObjects());
 					//The plto should have a parent filter
 					ASSERT(plotData->parent);
 					//The plot should have an index, so we can keep
@@ -1330,28 +1389,6 @@ unsigned int VisController::updateScene()
 						}
 						case VOXEL_REPRESENT_ISOSURF:
 						{
-#ifdef HPMC_GPU_ISOSURFACE
-							//GPU based isosurface
-							DrawIsoSurfaceWithShader *dS = new DrawIsoSurfaceWithShader;
-							
-
-							if(dS->canRun())
-							{
-								dS->init(*v);
-
-								dS->wantsLight=true;
-//								dS->setColour(vSrc->r,vSrc->g,
-//										vSrc->b,1.0);
-								dS->setScalarThresh(vSrc->isoLevel);
-								targetScene->addDrawable(dS);
-								//Don't process normal isosurf.
-								//code.
-								break;
-							}
-					
-							delete dS;
-#else
-
 							DrawIsoSurface *d = new DrawIsoSurface;
 
 							d->swapVoxels(v);
@@ -1362,7 +1399,6 @@ unsigned int VisController::updateScene()
 							d->wantsLight=true;
 
 							targetScene->addDrawable(d);
-#endif
 							break;
 						}
 						default:
@@ -1379,6 +1415,9 @@ unsigned int VisController::updateScene()
 
 				
 			}
+			
+			//Run the callback to update the window as needed
+			(*wxYieldCallback)();
 
 		}
 	
@@ -1409,7 +1448,7 @@ unsigned int VisController::updateScene()
 			unsigned int plotID;
 
 			//Retreive the uniqueID
-			l=(wxListUint*)plotSelList->GetClientData(ui);
+			l=(wxListUint*)plotSelList->GetClientObject(ui);
 			plotID = l->value;
 			if(targetPlots->isPlotVisible(plotID))
 				plotSelList->SetSelection(ui);
@@ -1518,16 +1557,16 @@ void VisController::checkRefreshValidity(const vector< const FilterStreamData *>
 
 		switch(f->getStreamType())
 		{
-		case STREAM_TYPE_IONS:
-		{
-			const IonStreamData *ionData;
-			ionData=((const IonStreamData *)f);
+			case STREAM_TYPE_IONS:
+			{
+				const IonStreamData *ionData;
+				ionData=((const IonStreamData *)f);
 
-		//	ASSERT(ionData->data.size());
-			break;
-		}
-		default:
-			;
+			//	ASSERT(ionData->data.size());
+				break;
+			}
+			default:
+				;
 		}
 
 	}
@@ -1540,19 +1579,59 @@ void VisController::checkRefreshValidity(const vector< const FilterStreamData *>
 		curData[ui]->cached == 0);
 	}
 
-	//Filter outputs for this filter should only be from those specified in filter emit mask
+	//Filter outputs for this filter should 
+	//	 -only be from those specified in filter emit mask
 	for(unsigned int ui=0; ui<curData.size(); ui++)
 	{
 		if(!curData[ui]->parent)
 		{
-			cerr << "Warning: orphan filter stream (FilterStreamData::parent == 0). This must be set when creating new filters in the ::refresh function for the filter. " << endl;
-			cerr << "Filter :"  << refreshFilter->getUserString() << "Stream Type: " << STREAM_NAMES[getBitNum(curData[ui]->getStreamType())] << endl;
+			cerr << "Warning: orphan filter stream (FilterStreamData::parent == 0)." <<
+				"This must be set when creating new filters in the ::refresh function for the filter. " << endl;
+			cerr << "Filter :"  << refreshFilter->getUserString() << "Stream Type: " << 
+				STREAM_NAMES[getBitNum(curData[ui]->getStreamType())] << endl;
 		}
-		if(curData[ui]->parent == refreshFilter)
+		else if(curData[ui]->parent == refreshFilter)
 		{
-			ASSERT(curData[ui]->getStreamType() & refreshFilter->getRefreshEmitMask());
+			//Check we emitted something that our parent's emit mask said we should
+			// by performing bitwise ops 
+			ASSERT(curData[ui]->getStreamType() & 
+				refreshFilter->getRefreshEmitMask());
 		}
 	}
+
+	//plot output streams should only have known types
+	//for various identfiers
+	for(unsigned int ui=0; ui<curData.size(); ui++)
+	{
+		if(curData[ui]->getStreamType() != STREAM_TYPE_PLOT)
+			continue;
+
+		PlotStreamData *p;
+		p =(PlotStreamData*)curData[ui];
+
+		//Must have valid trace style
+		ASSERT(p->plotType < PLOT_TRACE_ENDOFENUM);
+		//Must have valid error bar style
+		ASSERT(p->errDat.mode< PLOT_ERROR_ENDOFENUM);
+		//Must set the "index" for this plot -- required for 
+		ASSERT(p->index != (unsigned int)-1);
+		//Each region set must have valid IDs
+		ASSERT(p->regions.size() == p->regionID.size());
+	}
+	
+	//Voxel output streams should only have known types
+	for(unsigned int ui=0; ui<curData.size(); ui++)
+	{
+		if(curData[ui]->getStreamType() != STREAM_TYPE_VOXEL)
+			continue;
+
+		VoxelStreamData *p;
+		p =(VoxelStreamData*)curData[ui];
+
+		//Must have valid representation 
+		ASSERT(p->representationType< VOXEL_REPRESENT_END);
+	}
+
 }
 #endif
 
@@ -1968,6 +2047,17 @@ bool VisController::saveState(const char *cpFilename, std::map<string,string> &f
 
 	if(!f)
 		return false;
+	
+	//Write header, also use local language if available
+	const char *headerMessage = NTRANS("This file is a \"state\" file for the 3Depict program, and stores information about a particular analysis session. This file should be a valid \"XML\" file");
+
+	f << "<!--" <<  headerMessage;
+	if(TRANS(headerMessage) != headerMessage) 
+		f << endl << TRANS(headerMessage); 
+	
+	f << "-->" <<endl;
+
+
 
 	//Write state open tag 
 	f<< "<threeDepictstate>" << endl;
@@ -2219,7 +2309,8 @@ bool VisController::saveState(const char *cpFilename, std::map<string,string> &f
 	//Close XMl tag.	
 	f<< "</threeDepictstate>" << endl;
 
-
+	//Debug check to ensure we have written a valid xml file
+	ASSERT(isValidXML(cpFilename));
 
 	return true;
 }
@@ -2227,9 +2318,6 @@ bool VisController::saveState(const char *cpFilename, std::map<string,string> &f
 bool VisController::loadState(const char *cpFilename, std::ostream &errStream, bool merge) 
 {
 	//Load the state from an XML file
-	//FIXME: This could leave the viscontrol in a half
-	//  loaded state if the file was non-parseable.
-	//  Create disposable data structure to hold intermed. state
 	
 	//here we use libxml2's loading routines
 	//http://xmlsoft.org/
@@ -2299,22 +2387,31 @@ bool VisController::loadState(const char *cpFilename, std::ostream &errStream, b
 		{
 			xmlString=xmlGetProp(nodePtr, (const xmlChar *)"version"); 
 
-			vector<string> vecStrs;
-			vecStrs.push_back((char *)xmlString);
-			if(vecStrs[0].find_first_not_of("0123456789.")!= std::string::npos)
+			if(xmlString)
 			{
-				vecStrs.push_back(PROGRAM_VERSION);
-				if(getMaxVerStr(vecStrs)!=PROGRAM_VERSION)
+				string tmpVer;
+				
+				tmpVer =(char *)xmlString;
+				//Check to see if only contains 0-9 period and "-" characters (valid version number)
+				if(tmpVer.find_first_not_of("0123456789.-")== std::string::npos)
 				{
-					errStream << TRANS("State was created by a newer version of this program.. ")
-						<< TRANS("file reading will continue, but may fail.") << endl ;
+					//Check between the writer reported version, and the current program version
+					vector<string> vecStrs;
+					vecStrs.push_back(tmpVer);
+					vecStrs.push_back(PROGRAM_VERSION);
+
+					if(getMaxVerStr(vecStrs)!=PROGRAM_VERSION)
+					{
+						errStream << TRANS("State was created by a newer version of this program.. ")
+							<< TRANS("file reading will continue, but may fail.") << endl ;
+					}
 				}
+				else
+				{
+					errStream<< TRANS("Warning, unparseable version number in state file. File reading will continue, but may fail") << endl;
+				}
+				xmlFree(xmlString);
 			}
-			else
-			{
-				errStream<< TRANS("Warning, unparseable version number in state file. File reading will continue, but may fail") << endl;
-			}
-			xmlFree(xmlString);
 		}
 		else
 		{
@@ -2633,9 +2730,6 @@ bool VisController::loadState(const char *cpFilename, std::ostream &errStream, b
 		}
 	}
 
-
-
-
 	if(!merge)
 	{
 		std::swap(filters,newFilterTree);
@@ -2669,6 +2763,9 @@ bool VisController::loadState(const char *cpFilename, std::ostream &errStream, b
 			std::swap(filters,newFilterTree);
 	}
 
+	//NOTE: We must be careful about :return: at this point
+	// as we now have mixed in the new filter data.
+
 	//Re-generate the ID mapping for the stashes
 	stashUniqueIDs.clear();
 	for(unsigned int ui=0;ui<stashedFilters.size();ui++)
@@ -3107,8 +3204,10 @@ void VisController::updateRawGrid() const
 	unsigned int startCol;
 
 	//Clear the grid
-	targetRawGrid->DeleteCols(0,targetRawGrid->GetNumberCols());
-	targetRawGrid->DeleteRows(0,targetRawGrid->GetNumberRows());
+	if(targetRawGrid->GetNumberCols())
+		targetRawGrid->DeleteCols(0,targetRawGrid->GetNumberCols());
+	if(targetRawGrid->GetNumberRows())
+		targetRawGrid->DeleteRows(0,targetRawGrid->GetNumberRows());
 	
 	for(unsigned int ui=0;ui<plotData.size(); ui++)
 	{
@@ -3260,7 +3359,7 @@ void VisController::popRedoStack()
 void VisController::clearRedoStack()
 {
 	//Delete the undo stack trees
-	while(redoFilterStack.size())
+	while(!redoFilterStack.empty())
 	{
 		//clean up
 		for(tree<Filter * >::iterator it=redoFilterStack.back().begin(); 
